from src.opencities.authorization import authorize
from urllib import parse
import requests
import json


# Function to get, post, or delete files using the OC API.
def get_or_create_folder(path, admin_url, api_key, app_id, shared='false'):
    enc_path = parse.quote(path, safe='')
    enc_shared = parse.quote(shared, safe='')
    url = admin_url + '/api/v1/folder/GetOrCreateFolderByPath?path=' + enc_path + '&isShared=' + enc_shared
    auth = authorize(url, api_key, app_id)

    r = requests.get(url=url,
                     headers={'Accept': 'application/json',
                              'Authorization': auth}
                     )

    return r


def get_folder_files(folder_id, admin_url, api_key, app_id, file_name=None, shared="false"):
    api_method = 'GET'
    url = admin_url + '/api/v1/folder/GetFiles?FolderId=' + folder_id + '&isShared=' + shared

    if file_name:
        url = url + '&Name=' + file_name

    # Send request to OC API.
    auth = authorize(url, api_key, app_id)
    headers = {'Authorization': auth}

    response = requests.request(api_method,
                                url,
                                headers=headers)

    return response


def create_file(file, folder_id, admin_url, api_key, app_id, owner='admin'):
    api_method = 'POST'
    url = admin_url + '/api/v1/files/Create'
    image_types = ['gif', 'png', 'jpg', 'jpe', 'jpeg', 'tiff', 'tif', 'bmp', 'ico', 'svg']
    media_types = ['asf', ' asx', ' wm', ' wmx', ' wmp', ' wma', ' wax', ' wmv', ' wvx', ' avi', ' wav', ' mpeg',
                   ' mpg', ' mpe', ' mov', ' m1v', ' mp2', ' mpv2', ' mp2v', ' mpa', ' mp3', ' m3u', ' mid', ' midi',
                   ' rm', ' rma', ' rmi', ' rmv', ' aif', ' aifc', ' aiff', ' au', ' snd', ' flv', ' mp4', ' svg']
    file_name = file.split('/')[-1]
    extension = file_name.split('.')[-1]
    object_type = 'Document'
    auth = authorize(url, api_key, app_id, method=api_method)

    # Determine file and application type.
    if extension in media_types:
        object_type = 'Media'

    if extension in image_types:
        object_type = 'Image'

    # Build json body to send to API.
    payload = {
        'json': json.dumps(
            {
                "FolderId": folder_id,
                "FileObjectType": object_type,
                "Owner": owner,
                "Name": file_name
            }
        )
    }

    files = [
        (
            'file', (
                file_name,
                open(file, 'rb'),
                ''
            )
        )
    ]

    headers = {
        'Authorization': auth
    }

    response = requests.request(
        api_method,
        url,
        headers=headers,
        data=payload,
        files=files
    )

    return response


def updated_file(file, folder_id, admin_url, api_key, app_id, owner='admin'):
    api_method = 'POST'
    url = admin_url + '/api/v1/files/Create'
    image_types = ['gif', 'png', 'jpg', 'jpe', 'jpeg', 'tiff', 'tif', 'bmp', 'ico', 'svg']
    media_types = ['asf', ' asx', ' wm', ' wmx', ' wmp', ' wma', ' wax', ' wmv', ' wvx', ' avi', ' wav', ' mpeg',
                   ' mpg', ' mpe', ' mov', ' m1v', ' mp2', ' mpv2', ' mp2v', ' mpa', ' mp3', ' m3u', ' mid', ' midi',
                   ' rm', ' rma', ' rmi', ' rmv', ' aif', ' aifc', ' aiff', ' au', ' snd', ' flv', ' mp4', ' svg']
    file_name = file.split('/')[-1]
    extension = file_name.split('.')[-1]
    object_type = 'Document'
    auth = authorize(url, api_key, app_id, method=api_method)

    # Determine file and application type.
    if extension in media_types:
        object_type = 'Media'

    if extension in image_types:
        object_type = 'Image'

    # Build json body to send to API.
    payload = {
        'json': json.dumps(
            {
                "FolderId": folder_id,
                "FileObjectType": object_type,
                "Owner": owner,
                "Name": file_name
            }
        )
    }

    files = [
        (
            'file', (
                file_name,
                open(file, 'rb'),
                ''
            )
        )
    ]

    headers = {
        'Authorization': auth
    }

    response = requests.request(
        api_method,
        url,
        headers=headers,
        data=payload,
        files=files
    )

    return response


def create_or_update_file(file, folder_id, admin_url, api_key, app_id):
    file_name = file.split('/')[-1]
    # Check if file already exists.
    exists = False
    file_check = get_folder_files(folder_id, admin_url, api_key, app_id, file_name=file_name)
    file_check = file_check.json()
    exist_files = [x['Name'] for x in file_check]

    if file_name in exist_files:
        exists = True

    # Create the file using the OC API.
    if exists:
        response = updated_file(file,
                                folder_id,
                                admin_url,
                                api_key,
                                app_id)
    else:
        response = create_file(file,
                               folder_id,
                               admin_url,
                               api_key,
                               app_id)

    return response
